/* myR - A C++ library for accessing R
 *
 * (C) 2004 Pedro Ribeiro de Andrade Neto
 * pedrorib@yahoo.com
 */

#include "myR.h"
#include <stdio.h>

//namespace myRnamespace
//{

extern "C"
{

#include "/home/paulojus/DOWNLOADS/R/R/src/include/Startup.h"
	
extern int Rf_initEmbeddedR(int argc, char **argv);

void myR::myCallBack()
{
	printf("callback\n");
}

//typedef structRstart *Rstart;

myR::myR()
{
    char* argv[] = {"myRembedded"};
    Rf_initEmbeddedR(1, argv);
    structRstart rp;
    Rstart Rp = &rp;
    R_DefParams(Rp);
//    Rp->Callback = myCallBack; // dont have Callback in Rstart
    R_SetParams(Rp);
}

// the first argv value must be the name of the aplication
myR::myR(int argc, char** argv)
{
    Rf_initEmbeddedR(argc, argv);
}

SEXP myR::parseString(const char *s, int *parts, ParseStatus *status)
{
  int maxParts = 1;
  char *c = (char*) s;
  SEXP cv, pr;

  while (*c)
  {
    if (*c=='\n' || *c==';') maxParts++;
    c++;
  }

  PROTECT( cv = allocVector(STRSXP, 1) );
  SET_VECTOR_ELT( cv, 0, mkChar(s) );

  while (maxParts > 0)
  {
    pr = R_ParseVector(cv, maxParts, status);
    if (*status != PARSE_INCOMPLETE && *status != PARSE_EOF) break;
    maxParts--;
  }

  UNPROTECT(1);
  *parts = maxParts;

  return pr;
}

SEXP myR::evaluate(SEXP ps, int& Rerror)
{
    SEXP exp = R_NilValue;

    if (TYPEOF(ps) == EXPRSXP && LENGTH(ps) > 0)
    {
        int bi = 0;
        while ( bi < LENGTH(ps) )
        {
            SEXP pxp = VECTOR_ELT(ps, bi);
            Rerror = 0;
            exp = R_tryEval(pxp, R_GlobalEnv, &Rerror);
            bi++;
            if (Rerror) break;
        }
    } else
    {
        Rerror = 0;
        exp = R_tryEval(ps, R_GlobalEnv, &Rerror);
    }
    return exp;
}

myError myR::Execute(const char* command, SEXP& result)
{
    int parts;
    ParseStatus status;
    SEXP ps;
    int error;

    ps = parseString(command, &parts, &status);
    if (!ps) return myERROR_PARSING;

    result = evaluate(ps, error);

    return error? myERROR_EVALUATING : myOK;
}

void myR::DeclareVar(const char* name, SEXP var)
{
    SEXP var_name = Rf_install(name);

    Rf_defineVar(var_name, var, R_GlobalEnv);

    PROTECT(var);
}

void myR::DeleteVar(SEXP var)
{
    UNPROTECT_PTR(var);
}

void myR::printSEXP(SEXP e)
{
    int i = 0;

    switch( TYPEOF(e) )
    {
        case NILSXP:
            printf("NULL value\n");
            return;

        case LANGSXP:
            printf("language construct\n");
            return;

        case REALSXP:
            if (LENGTH(e) > 1)
            {
                printf("Vector of real variables: ");
                while( i < LENGTH(e) )
                {
                    printf("%f",REAL(e)[i]);
		    if ( !(REAL(e)[i] < 0) && !(REAL(e)[i] >= 0) ) printf("__NAN__");
                    if (i < LENGTH(e) - 1) printf(", ");
                    if (dumpLimit && i > dumpLimit) { printf("..."); break; }
                    i++;
                }
                putchar('\n');
            }
            else printf( "Real variable %f\n", *REAL(e) );
            return;
        case EXPRSXP:
            printf( "Vector of %d expressions:\n", LENGTH(e) );
            while( i < LENGTH(e) )
            {
                if (dumpLimit && i > dumpLimit){ printf("...");  break; }
                printSEXP( VECTOR_ELT(e,i) );
                i++;
            }
            return;

         case INTSXP:
            printf( "Vector of %d integers:\n", LENGTH(e) );

            while( i < LENGTH(e) )
            {
                if (dumpLimit && i > dumpLimit) { printf("..."); break; }
                printf("%d",INTEGER(e)[i]);
                if (i < LENGTH(e) - 1) printf(", ");
                i++;
            };
            putchar('\n');
            return;

        case VECSXP:
            printf( "Vector of %d fields:\n", LENGTH(e) );
            while( i < LENGTH(e) )
            {
                if (dumpLimit && i>dumpLimit) { printf("..."); break; };
                printSEXP(VECTOR_ELT(e,i));
                i++;
            }
            return;

        case STRSXP:
            printf( "String vector of length %d:\n", LENGTH(e) );
            while( i < LENGTH(e) )
            {
                if (dumpLimit && i > dumpLimit) { printf("..."); break; };
                printSEXP(VECTOR_ELT(e,i)); i++;
            }
            return;

        case CHARSXP:
            printf( "scalar string: \"%s\"\n",(char*) STRING_PTR(e) );
            return;

        case SYMSXP:
            printf( "Symbol, name: "); printSEXP(PRINTNAME(e) );
            return;

        default:
            printf( "Unknown type: %d\n", TYPEOF(e) );
    }
};

}

//}

